<table>
  {% for chapter in chapters %}
  <tr>
    <td>
      <a href="./{{ chapter.chapter_number }}/">
        {{ chapter.chapter_number }}
      </a>
    </td>
    <td>{{ chapter.title }}</td>
  </tr>
  {% endfor %}
</table>
