<form action="/upload" method="post" enctype="multipart/form-data">
  <input type="file" name="uploaded_files" multiple />
  <button type="submit">Upload</button>
</form>

<table>
  {% for ebook in ebooks %}
  <tr>
    <td>
      <a href="./{{ ebook.id }}/"> {{ ebook.id }} </a>
    </td>
    <td>{{ ebook.title }}</td>
  </tr>
  {% endfor %}
</table>
