import typing
import datetime


class Chapter(typing.NamedTuple):
    title: str
    paragraphs: typing.Iterable[str]


class Book(typing.NamedTuple):
    title: str
    chapters: typing.Iterable[Chapter]


class BookRecord(typing.NamedTuple):
    id: int
    date_added: datetime.datetime
    title: str


class ChapterRecord(typing.NamedTuple):
    chapter_number: int
    title: str
